import { MediaPlayer } from 'dashjs';
import queryString from 'query-string';

import style from './player.styl';

// Nonce used by Webpack for CSP
// eslint-disable-next-line
const __webpack_nonce__ = 'uGo4I9ydb2hP393boc/24Vu7diUk/Mf84w9khcZkynk=';

window.addEventListener('load', () => {
  const statusContainer = document.createElement('div');
  statusContainer.classList.add(style.status);
  const statusContent = document.createElement('span');
  statusContent.classList.add(style.statusInner);
  statusContainer.appendChild(statusContent);
  document.body.appendChild(statusContainer);

  function showStatus() {
    if (!(style['status--visible'] in statusContainer.classList)) {
      statusContainer.classList.add(style['status--visible']);
    }
  }

  function setStatus(text) {
    statusContent.innerText = text;
  }

  function hideStatus() {
    statusContainer.classList.remove(style['status--visible']);
  }

  showStatus('Constructing your clock...');

  const playerContainer = document.createElement('video');
  playerContainer.classList.add(style.video);
  playerContainer.loop = true;
  playerContainer.controls = false;

  const parsedHash = queryString.parse(window.location.query || window.location.hash);
  let secondsOfDay;
  if (parsedHash.s !== undefined) {
    // use query-provided start time
    secondsOfDay = parseInt(parsedHash.s, 10);
  } else {
    // calculate start time
    const date = new Date();
    const localTime = date.getTime() - (1000 * 60 * date.getTimezoneOffset());
    secondsOfDay = (localTime % (1000 * 60 * 60 * 24)) / 1000;
  }

  // initialize video
  const player = MediaPlayer().create();
  const url = `video/clock.mpd#s=${secondsOfDay}`;
  const autoplay = true;
  player.initialize(playerContainer, url, autoplay);

  player.on('error', (e) => {
    showStatus();
    setStatus(`Error: ${e}`);
  });

  player.on('bufferEmpty', () => {
    setStatus('We ran out of wood!');
  });

  player.on('bufferStalled', () => {
    setStatus('We are waiting for more wood…');
  });

  player.on('bufferLoaded', () => {
    setStatus('Almost there…');
  });

  player.on('playbackWaiting', () => {
    showStatus();
    setStatus('Waiting for the workers to arrive…');
  });

  player.on('playbackPlaying', () => {
    hideStatus();
    document.body.appendChild(playerContainer);
  });
});
